#!/usr/bin/env python

"""Tests for `datagrep` package."""

import pytest

from click.testing import CliRunner
# from collections import defaultdict
# import numpy as np
import sys

from datagrep.agents import Agent
from datagrep.environments import Environment
from datagrep import cli


@pytest.fixture
def response():
    """Sample pytest fixture.

    See more at: http://doc.pytest.org/en/latest/fixture.html
    """
    # import requests
    # return requests.get('https://github.com/audreyr/cookiecutter-pypackage')


def test_content(response):
    """Sample pytest test function with the pytest fixture as an argument."""
    # from bs4 import BeautifulSoup
    # assert 'GitHub' in BeautifulSoup(response.content).title.string


def test_monte_carlo_exploration():
    """Copied from Udacity's Deep RL Nanodegree Monte_Carlo notebook.

    https://github.com/mjschock/deep-reinforcement-learning/blob/master/monte-carlo/Monte_Carlo.ipynb
    """
    env = Environment("Blackjack-v0")
    agent = Agent()

    for i_episode in range(3):
        env.reset()
        observation, available_actions, available_action_probabilities = agent.sense(
            env
        )
        assert available_actions == [0, 1]
        assert available_action_probabilities == [0.5, 0.5]

        while True:
            (
                observation,
                available_actions,
                available_action_probabilities,
            ) = agent.sense(env)
            action, reward, done = agent.act(env, observation, available_actions)
            assert available_actions == [0, 1]
            assert available_action_probabilities == [0.5, 0.5]

            if done:
                assert reward in (-1.0, 0.0, 1.0)
                break


def test_monte_carlo_prediction():
    """Copied from Udacity's Deep RL Nanodegree Monte_Carlo notebook.

    https://github.com/mjschock/deep-reinforcement-learning/blob/master/monte-carlo/Monte_Carlo.ipynb
    """
    env = Environment("Blackjack-v0")

    def observation_wrapper(observation, available_actions):
        available_action_probabilities = (
            [0.8, 0.2] if observation[0] > 18 else [0.2, 0.8]
        )
        return observation, available_actions, available_action_probabilities

    def generate_episode_from_limit_stochastic(bj_env):
        episode = []
        bj_env.reset()

        while True:
            (
                observation,
                available_actions,
                available_action_probabilities,
            ) = agent.sense(env)

            action, reward, done = agent.act(
                env, observation, available_actions, available_action_probabilities
            )

            episode.append((observation, action, reward))

            if done:
                break

        return episode

    agent = Agent(observation_wrapper=observation_wrapper)

    def mc_prediction_q(env, num_episodes, generate_episode, gamma=1.0):
        # loop over episodes
        for i_episode in range(1, num_episodes + 1):
            # monitor progress
            if i_episode % 1000 == 0:
                print("\rEpisode {}/{}.".format(i_episode, num_episodes), end="")
                sys.stdout.flush()

            episode = generate_episode(env)
            agent.learn([episode], gamma)

        return agent.get_quality_function()

    q = mc_prediction_q(env, 50, generate_episode_from_limit_stochastic)

    assert q.predict_action_value((0, 0, 0), 0) == 0.0


def test_command_line_interface():
    """Test the CLI."""
    runner = CliRunner()
    result = runner.invoke(cli.main)
    assert result.exit_code == 0
    assert "datagrep.cli.main" in result.output
    help_result = runner.invoke(cli.main, ["--help"])
    assert help_result.exit_code == 0
    assert "--help  Show this message and exit." in help_result.output
